#!/usr/bin/env python3
# you schould not need to do anything in here
import sys                           #needed for executing this sript in diverent modes
import subprocess as sp              #needed to run mcstas
import argparse
import os
from importlib import import_module
from mcpw.mcstas_wrapper import run_mcstas, run_compiler, run_instrument,\
                            valid_config, valid_mcconfig,\
                            psave, pload,\
                            check_for_detector_output, get_result_path_from_input

#for exeption handling
class ArgumentParserError(Exception):
    pass
class ThrowingArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        raise ArgumentParserError(message)


def main():
    # defining the argument parser for nice usage
    parser= ThrowingArgumentParser(description="Control command for automatic mcstas usage for the Reseda Instrument")

    subparsers = parser.add_subparsers(title="modes", required=True,
             description="Use 'manager mode --help' to view the help for any command.",
             metavar='mode')

    parser.add_argument('-p', '--python_file', dest='python_file', default='', type=str, required=True,
                       help='path (absolute or not) to the python file containing mcvariables and analyse functions')

    parser.add_argument('-d', '--result-dir', dest='result_dir', default='', type=str,
                       help='directory name for the simulationresults. If none is given or foulder name exists allrady, a increment Folder Name is generated')

    parser_server = subparsers.add_parser('server', help='mcstas will be executed localy and the processed simulation results packed in a tarball')
    parser_server.set_defaults(func='server')

    parser_remote = subparsers.add_parser('remote', help='mcstas will be executed on a remote machine')
    parser_remote.set_defaults(func='remote')

    parser_local = subparsers.add_parser('local', help='mcstas will be executed on localy')
    parser_local.set_defaults(func='local')

    parser_full = subparsers.add_parser('full', help='mcstas will be executed on localy and the analyze function will be called after')
    parser_full.set_defaults(func='full')

    parser_analyse = subparsers.add_parser('analyse', help='analyse function will be called')
    parser_analyse.set_defaults(func='analyse')



    #retreiveng command arguments
    try:
        args = parser.parse_args()
    except ArgumentParserError as e:
        parser.print_help()
        print(f"\n{e}")
        exit(1)


    # importing var and mcvar from local_var.py and reseda.py files
    sys.path.append(os.getcwd())
    sys.path.append(os.path.dirname(args.python_file))
    from local_var import variables

    var = variables()
    valid_config(var)


    # adding local working directory to PATH
    sys.path.append(var.p_local)
    # adding pyhton file location to PATH
    #importing python_file as module
    pyinstr = import_module(f"{os.path.basename(args.python_file).split('.')[0]}")

    # pulling relevant functions and classes from python file
    mcvariables = vars(pyinstr)['mcvariables']
    analyse = vars(pyinstr)['analyse']
    post_mcrun_funktions = vars(pyinstr)['post_mcrun_funktions']
    # initializing mcvariables class
    mcvar = mcvariables()

    msg  = ''
    #calling the correct function according to command arguments
    if 'func' not in args:
        # Fallback to --help.
        parser.print_help()
    else:
        (mcvar.dn,msg) = get_result_path_from_input(var, mcvar, msg, args) # logic for retreiveng the correct name for the result foulder
        valid_mcconfig(var,mcvar)
        if args.func == 'analyse':
            mcvar = pload(var.sim_res/mcvar.dn/'variables') #loading the correct variables
            valid_mcconfig(var,mcvar)
            check_for_detector_output(var,mcvar)
            analyse(var, mcvar, msg)                                               #call analyse, defined in reseda.py
            exit(0)
        if args.func == 'server':#this runs the script in server mde
            run_mcstas(var,mcvar)
            run_compiler(var,mcvar)
            run_instrument(var,mcvar)
            check_for_detector_output(var,mcvar)
            psave(mcvar, var.sim_res/mcvar.dn/'variables')  #save mcstas variables
            post_mcrun_funktions(var, mcvar, msg) # contains functions that get executed after mcstas finished and can i.e. reformate the output
            sp.run(['tar', '-cf' '{}/{}.tar'.format(var.sim_res, mcvar.dn), var.sim_res/mcvar.dn]) #compress data

        elif args.func == 'remote':#use this if you want to run the simulation on a remote machine (setup has to be done beforhand)
            sp.run(['scp', '-r', '-P', str(var.port), var.instr_file, '{}:{}'.format(var.server, var.p_server)])#copy mcstas-instrument to remote
            sp.run(['scp', '-r', '-P', str(var.port), 'manager.py', '{}:{}'.format(var.server, var.p_server)])#copy this file to remote
            sp.run(['scp', '-r', '-P', str(var.port), 'reseda.py', '{}:{}'.format(var.server, var.p_server)])#copy this file to remote
            sp.run(['ssh' , '-p', str(var.port), var.server, 'cd {}; python {}manager.py server {}'.format(var.p_server, var.p_server,mcvar.dn)])#run this file with server atribute remote
            sp.run(['scp', '-l', str(var.rate), '-r', '-P', str(var.port), '{}:{}.tar'.format(var.server, var.p_server/var.sim_res/mcvar.dn), var.p_local])#download data from remote
            sp.run(['tar', '-xf', '{}.tar'.format(mcvar.dn)])#decompress data

        elif args.func == 'local':#use this to run the script localy
            run_mcstas(var,mcvar)
            run_compiler(var,mcvar)
            run_instrument(var,mcvar)
            check_for_detector_output(var,mcvar)
            psave(mcvar, var.sim_res/mcvar.dn/'variables')  #save mcstas variables
            post_mcrun_funktions(var, mcvar, msg) # contains functions that get executed after mcstas finished and can i.e. reformate the output

        elif args.func == 'full':
            run_mcstas(var,mcvar)
            run_compiler(var,mcvar)
            run_instrument(var,mcvar)
            check_for_detector_output(var,mcvar)
            psave(mcvar, var.sim_res/mcvar.dn/'variables')  #save mcstas variables
            post_mcrun_funktions(var, mcvar, msg) # contains functions that get executed after mcstas finished and can i.e. reformate the output
            analyse(var, mcvar, msg)                                               #call analyse, defined in reseda.py
        print(msg)

if __name__ == '__main__':
    main()



'''
class variables():
    def __init__(self):
        #directorys
        self.p_server    = Path("/path/to/working/dir")
        self.p_local     = Path("/path/to/working/dir")
        self.sim_res     = "simulation_results/"
        #ssh related variables
        self.rate        = 32000 #scp transfairrate in bits/s
        self.port        = 22
        self.server      = ""
        #mcstas location
        self.mcstas  = "mcstas"
        #mcstas variables
        self.mpi         = 2
        #additional c compiler flags
        self.cflags      = ""
'''
def create_local_var_lines(args):
    print("generating class variables")
    class_variables_lines = []
    class_variables_lines.append("from pathlib import Path             #needed for the path logic")
    class_variables_lines.append("class variables():")
    class_variables_lines.append("    def __init__(self):")
    class_variables_lines.append("        #directorys")
    class_variables_lines.append(f'        self.p_server     = Path("/")')
    class_variables_lines.append(f'        self.p_local      = Path("{args.working_dir}")')
    class_variables_lines.append(f'        self.sim_res      = "{args.output_dir}"')
    class_variables_lines.append(f"        #ssh related variables")
    class_variables_lines.append(f'        self.rate         = 32000 #scp transfairrate in bits/s')
    class_variables_lines.append(f'        self.port         = 22')
    class_variables_lines.append(f'        self.server       = ""')
    class_variables_lines.append(f"        #mcstas location")
    class_variables_lines.append(f'        self.mcstas       = "{args.mcstas}"')
    class_variables_lines.append(f'        self.componentdir = "{args.component_dir}"')
    class_variables_lines.append(f"        #mcstas variables")
    class_variables_lines.append(f'        self.mpi          = {args.mpi}')
    class_variables_lines.append(f"        #additional c compiler flags")
    class_variables_lines.append(f'        self.cflags       = ""')

    return class_variables_lines

def create_local_var(args):
    class_variables_lines = create_local_var_lines(args)
    with open(f"{args.working_dir}/local_var.py", "w") as pyfile:
        for line in class_variables_lines:
            print(line)
            pyfile.write("{}\n".format(line))

"""
This Script creats a python class contianing all variables,
defined in the DEFINE INSTRUMENT Section of an McStas Instrument File.
The Define Section must have following Format:
---------------------------------------------
DEFINE INSTRUMENT 'instrument'
(
    double x = 1.0,     //comment

    int i = 1,          //comment
    float f = 1.0,      //comment
)
---------------------------------------------
empty lines will be ignored

The Outputformat will be:
---------------------------------------------
class mcvariables():#class to hold the variables needed to run the mcstas simulation
    def __init__(self):
        # allways needed
        self.dn             = 'run1' #name of the result diretory for the run
        self.n              = 1000000 #numbers of neutrons for simulation
        self.N              = 1       #steps (see sweep)
        self.instr_file     = "test.instr" #the name of the instrument file, must be located in p_server/p_local, all custom components used by the instrument must be located in the same diretory
        #
        #self.sweep          = Sweep(-0.1,0.1,'A')
        # variables defined in the DEFINE INSTRUMENT section of the mcstas instrument
        x = 1.0,        //comment
        i = 1,          //comment
        f = 1.0,        //comment

post_mcrun_funktions(var, mcvar):
    func1(var, mcvar)
    func2(var, mcvar)

analyse(var,mcvar):
    func1(var, mcvar)
    ...
    funcX(var,mcvar)

---------------------------------------------
"""
import sys
from os.path import isfile, basename
def check_args(args):
    if not args.instrument.endswith(".instr"):
        sys.exit(f"{args.instrument} dose not end on .instr")
    if not isfile(args.working_dir +'/'+ args.instrument):
        sys.exit(f" the instrument file '{args.instrument}' is not located in the working directory '{args.working_dir}'")



def create_class_mcvariables_lines(instrument):
    var_lines = []
    with open(instrument) as mcfile:
        befor_define_section = True
        in_define_section = False
        for line in mcfile:
            if befor_define_section:
                if line.startswith("DEFINE INSTRUMENT"):
                    befor_define_section = False
                    in_define_section = True
            if in_define_section:
                if line == ")\n":
                    in_define_section = False
                if (line != "(\n") and not line.startswith("DEFINE INSTRUMENT") and line !="\n" and line !=")\n":
                    var_lines.append(line.replace("\t","    ").replace("\n","").replace(","," ").replace("//","#").lstrip().split(' ',1)[1])

    class_mcvariables_lines = []
    class_mcvariables_lines.append("class mcvariables():#class to hold the variables needed to run the mcstas simulation")
    class_mcvariables_lines.append("    def __init__(self):")
    class_mcvariables_lines.append("        # allways needed")
    class_mcvariables_lines.append('        self.dn             = "default"')
    class_mcvariables_lines.append("        self.n              = 1000000")
    class_mcvariables_lines.append(f'        self.instr_file     = "{basename(instrument)}"  #the name of the instrument file, must be located in p_server/p_local')
    class_mcvariables_lines.append("        self.scan           = Scan(-0.1,0.1,'A', 3) # (begining, ending, Unit, number of steps)")
    class_mcvariables_lines.append("        #variables defined in the DEFINE INSTRUMENT section of the mcstas instrument")
    for line in var_lines:
        if line.__contains__("="):
            class_mcvariables_lines.append(f"        self.{line}")
        else:
            class_mcvariables_lines.append(f"        # {line}")

    return class_mcvariables_lines

def create_header_lines():
    header_lines = []
    header_lines.append(f'# import section')
    header_lines.append(f'from mcpw.mcstas_wrapper import Scan')
    header_lines.append(f'')
    header_lines.append(f'#this file must allways contain:')
    header_lines.append(f'#class mcvariables() a class containing all parameters for the mcstas instrument file')
    header_lines.append(f'#def post_mcrun_funktions(var,mcvar) a funcition containing all functions that should be exectuded directly after mcstas has been executed')
    header_lines.append(f'#def analyse(var,mcvar) a funcition containing all functions that should be exectuded to analyse a finished mcstas simulation')
    return header_lines

def create_main_lines():
    main_lines = []
    main_lines.append(f'###########################')
    main_lines.append(f'#   your custom function  #')
    main_lines.append(f'###########################')
    main_lines.append(f'')
    main_lines.append(f'def custom_function1(var,mcvar,msg):')
    main_lines.append(f'    print("hallo world 1")')
    main_lines.append(f'')
    main_lines.append(f'def custom_function2(var,mcvar,msg):')
    main_lines.append(f'    print("hallo world 2")')
    main_lines.append(f'')
    main_lines.append(f'')
    main_lines.append(f'##########################################################')
    main_lines.append(f'# adding custom function to corresponding function group #')
    main_lines.append(f'##########################################################')
    main_lines.append(f'')
    main_lines.append(f'# adding functions to post mcstas function group')
    main_lines.append(f'def post_mcrun_funktions(var,mcvar,msg):')
    main_lines.append(f'    custom_function1(var,mcvar,msg)')
    main_lines.append(f'')
    main_lines.append(f'# adding functions to analyse function group')
    main_lines.append(f'def analyse(var,mcvar,msg):')
    main_lines.append(f'    custom_function2(var,mcvar,msg)')
    main_lines.append(f'')
    main_lines.append(f'# end of documentation')

    return main_lines


def create_python_file(args):
    check_args(args)
    header_lines = create_header_lines()
    main_lines = create_main_lines()
    print(f"reading {instrument}")
    class_mcvariables_lines = create_class_mcvariables_lines(f"{args.working_dir}/{args.instrument}")

    with open(f"{args.working_dir}/{args.instrument.split('.')[0]}.py", "w") as pyfile:
        for line in header_lines:
            print(line)
            pyfile.write("{}\n".format(line))
        pyfile.write("\n")
        pyfile.write("\n")
        for line in class_mcvariables_lines:
            print(line)
            pyfile.write("{}\n".format(line))
        pyfile.write("\n")
        pyfile.write("\n")
        for line in main_lines:
            print(line)
            pyfile.write("{}\n".format(line))
















import subprocess as sp              #needed to run mcstas
import sys                           #needed to select program mode of this script
import pickle                        #needed to save mcstas variables for later use
import os
from os.path import isfile, isdir, isabs, dirname, basename, splitext, join, islink
import locale
from shutil import copyfile, which
class Scan():#helping class for easyer use of the scan funktionality of mcstas, needs start and stop value and the unit of the values as well the numbers of steps
    def __init__(self, start, stop, unit, N):
        self.start = start
        self.stop = stop
        self.range = stop-start
        self.unit = unit
        self.N = N
        self.mc = '{},{}'.format(start, stop)
        self.step = (stop-start) / (N-1)
    def absolute_value(self, n):
        return self.step * n + self.start

def execute(command, errormsg, successmsg, print_command=True):
    if print_command:
        print(f"runing: {command}")
    run_return = sp.run(command, shell=True, text=True, stdout=sp.PIPE, stderr=sp.PIPE)
    # check if the process was successfully
    if run_return.returncode != 0:
        print(run_return.stdout)
        print(run_return.stderr)
        sys.exit(errormsg)
    else:
        print(f"{successmsg}\n")

def valid_config(var):
    #check dirs
    if not isdir(var.p_local):
        sys.exit(f"the given local working directory '{var.p_local}' dose not exist")
    if not isdir(var.p_server):
        sys.exit(f"the given server working directory '{var.p_server}' dose not exist\n\
                 if you dont intend to use the ssh feature, just set it so valid path")
    if isabs(var.sim_res):
        if not isdir(var.sim_res):
            sys.exit(f"the given simulation result directory '{var.sim_res}' dose not exist")
    else:
        if not isdir(var.p_local/var.sim_res):
            os.mkdir(var.p_local/var.sim_res)
            print(f"created simulation result directory '{var.sim_res}' in the local working directory")
    if isabs(var.sim_res): #making shure var.sim_res is allways a absolute path
        pass
    else:
        var.sim_res = var.p_local/var.sim_res

    mcstas = which(var.mcstas)
    if not mcstas:
        sys.exit(f"\nMcStas is not installed or the Path is inocrrect: {var.mcstas}")
    if islink(mcstas):
        var.mcstas = os.readlink(mcstas)

def valid_mcconfig(var,mcvar):
    if not mcvar.instr_file.split('.')[1] == "instr":
        sys.exit(f"\nthe given instrument file has not the correct ending:\
                 \nexpected: {mcvar.instr_file.split('.')[0]}.instr\
                 \ngot: {mcvar.instr_file}\
                 \nplease make shure to give a valid instrument file")

    if not isfile(var.p_local/mcvar.instr_file):
        sys.exit(f"\nthe instrument file '{mcvar.instr_file}' dose not exist")

    if var.mpi == 0:
        # test if gcc exists
        if not which("gcc"):
            sys.exit(f"\ngcc is not installed. please install it")
    else:
        # test if mpicc exists
        if os.name=='nt':
            if not which("mpiexec"):
                sys.exit(f"\nmpiexec is not installed. please install Microsoft MPI form the microsoft website")
        else:
            if not which("mpicc"):
                sys.exit(f"\nmpicc is not installed. please install openmpi")


def encode_files_to_local_encoding(file_or_dir_list, output_dir):
    file_list = []
    for entry in file_or_dir_list:
        if entry != '':
            if isdir(entry):
                with os.scandir(entry) as it:
                    for e in it:
                        if e.name.endswith(('.comp','.instr','.h')) and e.is_file():
                            file_list.append(e.path)
            else:
                file_list.append(entry)
    for file in file_list:
        try:
            of = open(file, mode='r', encoding='utf-8')
            content = of.read()
            of.close()
            file_name = basename(file)
            nf = open(f"{output_dir}/{file_name}", mode='w', encoding=locale.getpreferredencoding())
            nf.write(content)
            nf.close()
        except UnicodeDecodeError:
            copyfile(file, output_dir/basename(file))
        except UnicodeEncodeError:
            print(f' sciping {basename(file)}')

# mcstas -t --verbose -o outputfile instrfile (-I componentdir)
def run_mcstas(var, mcvar):
    # create output name
    instr_c_file = mcvar.instr_file.split('.')[0] + ".c"
    temp_dir = 'temp_encoding'
    if os.name=='nt':
        os.environ['MCSTAS'] = dirname(var.mcstas)+'/../lib/'
        if not isdir(var.p_local/temp_dir):
            os.mkdir(var.p_local/temp_dir)
        encode_files_to_local_encoding([var.p_local/mcvar.instr_file, var.p_local, var.componentdir], var.p_local/temp_dir)
        run_string = f"{var.mcstas} -t --verbose -o {var.p_local/instr_c_file} {basename(mcvar.instr_file)} -I {var.p_local/temp_dir} "
        # exectue the run_string and capture the output
        print(f"runing: {run_string}")
        run_return = sp.run(run_string, shell=True, text=True, stdout=sp.PIPE, stderr=sp.PIPE, cwd=var.p_local/temp_dir)
        for file in os.listdir(f"{var.p_local/temp_dir}"):
            os.remove(f"{var.p_local/temp_dir/file}")
        os.rmdir(f"{var.p_local/temp_dir}")
    else:
        # create run_string
        run_string = f"{var.mcstas} -t --verbose -o {var.p_local/instr_c_file} {var.p_local/mcvar.instr_file} -I {var.p_local} "
        #if a extra Component Dir is given add the option to the compile command
        if var.componentdir != "":
            run_string = run_string + f"-I {var.componentdir} "
        # exectue the run_string and capture the output
        print(f"runing: {run_string}")
        run_return = sp.run(run_string, shell=True, text=True, stdout=sp.PIPE, stderr=sp.PIPE)
    # check if the process was successfull
    if run_return.returncode != 0:
        print(run_return.stdout)
        print(run_return.stderr)
        sys.exit("An error occurred while running McStas Compiler")
    else:
        print(f"McStas compiler done\n")

# gcc (mpicc) -o p_local/reseda.out p_local/reseda.c -lm (-DUSE_MPI) -g -O2 -lm -std=c99
def run_compiler(var,mcvar, cflags=""):
    instr_c_file = mcvar.instr_file.split('.')[0] + ".c"
    if os.name=='nt':
        instr_out_file = mcvar.instr_file.split('.')[0] + ".exe"
    else:
        instr_out_file = mcvar.instr_file.split('.')[0] + ".out"
    #check if mpi is enabled
    if var.mpi == 0:
        run_string = f"gcc "
    else:
        if os.name=='nt':
            run_string = f"{dirname(var.mcstas)}/mpicc.bat -DUSE_MPI "
        else:
            run_string = f"mpicc -DUSE_MPI "
    run_string = run_string + f"-o {var.p_local/instr_out_file} {var.p_local/instr_c_file} -lm -g -O2 -std=c99 {cflags}"
    # exectue the run_string and capture the output
    execute(run_string, "An error occurred while running the C Compiler", "C compiler done")

# (mpirun -np 2) instr.out -n -d var=value
def run_instrument(var,mcvar):
    errormsg = "The Simmulation Failed"
    successmsg = "The simmulation compleated successfully"
    if os.name=='nt':
        instr_out_file = mcvar.instr_file.split('.')[0] + ".exe"
    else:
        instr_out_file = mcvar.instr_file.split('.')[0] + ".out"
    params = ''
    scan_var = []
    res_list = []
    #check if mpi is enabled
    if var.mpi == 0:
        run_string = f"{var.p_local/instr_out_file} -n {str(mcvar.n)} "
    else:
        if os.name=='nt':
            run_string = f"mpiexec -np {var.mpi} {var.p_local/instr_out_file} -n {str(mcvar.n)} "
        else:
            run_string = f"mpirun -np {var.mpi} {var.p_local/instr_out_file} -n {str(mcvar.n)} "
    # parsing the parameters and checking if a scan is required
    for var_name, var_value in mcvar.__dict__.items():
        if not (var_name in ["scan", "n", "dn", "instr_file"]):
            if isinstance(var_value, Scan):
                scan_var = [var_name, var_value]
            else:
                params = params + f"{var_name}={var_value} "
    # scan or no scan
    if scan_var:
        #scan
        #creating main result directory
        os.mkdir(var.sim_res/mcvar.dn)
        #scanning all points
        print(f"running: {run_string} {scan_var[0]}={scan_var[1].mc} ")
        for i in range (scan_var[1].N):
            print(f"step: {scan_var[0]}={scan_var[1].absolute_value(i)}")
            i_params = params + f"{scan_var[0]}={scan_var[1].absolute_value(i)} "
            final_run_string = run_string + f"-d {str(var.sim_res/mcvar.dn/str(i))} {i_params} "

            execute(final_run_string, errormsg, successmsg, print_command=False)
            res_list.append(var.sim_res/mcvar.dn/str(i))
    else:
        final_run_string = run_string + f"-d {str(var.sim_res/mcvar.dn)} {params} "
        execute(final_run_string, errormsg, successmsg)
        res_list.append(var.sim_res/mcvar.dn)
    return res_list

def psave(obj, file_path):#saves the given object as a pickle dump in the given file (file gets created)
    f = open(file_path, mode='xb')
    pickle.dump(obj, f)
    f.close

def pload(file_path):#funktion can read file writen by the psave function and returns its content
    f = open(file_path, mode='rb')
    obj = pickle.load(f)
    f.close
    return obj

def check_scan(var, mcvar, msg): #ignore for now, is something i might implement later fully
    dir_num = len(os.listdir(var.sim_res/mcvar.dn))
    if dir_num-4 == mcvar.scan.N:
        return True
    else:
        msg = msg + f"the number of result folders dont correspont to the number of steps (#Dir:{dir_num} vs scan.N:{mcvar.scan.N})\n"
        return False

def is_scan(mcvar):
    for var_name, var_value in mcvar.__dict__.items():
        if not var_name == "scan" and isinstance(var_value,Scan):
            return True
    return False

def check_for_detector_output(var, mcvar):
    if is_scan(mcvar):
        for i in range(mcvar.scan.N):
            if not os.path.isdir(var.sim_res/mcvar.dn/str(i)):
                print(f"the mcstas output dir {mcvar.dn}/{i} dose not exist.\nexiting")
                exit()
    else:
        if not os.path.isdir(var.sim_res/mcvar.dn):
            print(f"the mcstas output dir {mcvar.dn} dose not exist.\n exiting")
            exit()
        else:
            #print("all fine")
            return

def get_result_path_from_input(var, mcvar, msg, args):# logic for retreiveng the correct name for the result foulder
    if args.func == 'analyse':
        if args.result_dir:
            return args.result_dir, msg
        else:
            d = var.sim_res
            return sorted(d.iterdir(), key=os.path.getmtime, reverse=True)[0].name, msg
    if not args.result_dir:
        name = mcvar.dn
    else:
        name = args.result_dir
    if os.path.isdir(var.sim_res/name):
            counter = 0
            new_name = name + "_" + str(counter)
            while os.path.isdir(var.sim_res/new_name):
                counter = counter + 1
                new_name = name + "_" + str(counter)
            msg = f'####################\n new result directory is {new_name}\n####################\n'
            return new_name, msg
    return name, msg

def mcplot(var,mcvar,msg='', mode=''):
    if os.name == 'nt':
        mcplot_dir = f"{dirname(var.mcstas)}/../lib/tools/Python/mcplot"
    else:
        mcplot_dir= f"{dirname(var.mcstas)}/../tools/Python/mcplot"
    if mode == 'qt':
        run_string= f"{mcplot_dir}/pyqtgraph/mcplot.py "
    else:
        run_string= f"{mcplot_dir}/matplotlib/mcplot.py "

    if is_scan(mcvar):
        print('mcplot for a Scann not jet implemented')
        return
    else:
        run_string=run_string+f"{var.sim_res/mcvar.dn}"
    print(run_string)
    sp.run(run_string, shell=True)




# you schould not need to do anything in here
import sys                           #needed for executing this sript in diverent modes
import subprocess as sp              #needed to run mcstas
import argparse
import os
import re
from os.path import isdir, isfile, isabs
from mcpw.setup_tools import create_local_var, create_class_mcvariables_lines
from mcpw.mcstas_wrapper import run_mcstas, run_compiler,\
                                run_instrument, is_scan,\
                                check_for_detector_output,\
                                psave, pload, which, valid_config

def load_mcvariables(var, dn=''): # loads used parameters from simulation
    if not dn:
        print('no result folder name given.\n please enter one as 2nd argument to this function.')
        return
    return pload(var.sim_res/dn/'variables') #loading the correct variables

def simulate(var, mcvar, dn='', remote=False): #spawns a simulation if dn dose not jet exists and returns a list of result dirs
    msg = ''
    if not dn:
        print('no result folder name given.\n please enter one as 4th argument to this function.')
        return
    if os.path.isdir(var.sim_res/dn):
        print('A Simulation with this result foder name allrady exists.\n Skip Simulation.')
        res_list = []
        if is_scan(mcvar):
            for i in range(mcvar.scan.N):
                res_list.append(var.sim_res/mcvar.dn/str(i))
        else:
            res_list.append(var.sim_res/mcvar.dn)
        return load_mcvariables(var, dn), res_list
    else:
        mcvar.dn = dn
    #print(f'mcvar.dn={mcvar.dn}')
    if remote:#use this if you want to run the simulation on a remote machine (setup has to be done beforhand)
        sp.run(['scp', '-r', '-P', str(var.port), var.instr_file, '{}:{}'.format(var.server, var.p_server)])#copy mcstas-instrument to remote
        sp.run(['scp', '-r', '-P', str(var.port), 'manager.py', '{}:{}'.format(var.server, var.p_server)])#copy this file to remote
        sp.run(['scp', '-r', '-P', str(var.port), 'reseda.py', '{}:{}'.format(var.server, var.p_server)])#copy this file to remote
        sp.run(['ssh' , '-p', str(var.port), var.server, 'cd {}; python {}manager.py server {}'.format(var.p_server, var.p_server,mcvar.dn)])#run this file with server atribute remote
        sp.run(['scp', '-l', str(var.rate), '-r', '-P', str(var.port), '{}:{}.tar'.format(var.server, var.p/var.p_server/var.sim_res/mcvar.dn), var.p_local])#download data from remote
        sp.run(['tar', '-xf', '{}.tar'.format(mcvar.dn)])#decompress data
        print('remote simulation successfully\n')
        res_list = []
        if is_scan(mcvar):
            for i in range(mcvar.scan.N):
                res_list.append(var.sim_res/mcvar.dn/str(i))
        else:
            res_list.append(var.sim_res/mcvar.dn)
        return mcvar, res_list

    else:#use this to run the script localy
        run_mcstas(var,mcvar)
        run_compiler(var,mcvar)
        res = run_instrument(var,mcvar)
        check_for_detector_output(var,mcvar)
        psave(mcvar, var.sim_res/mcvar.dn/'variables')  #save mcstas variables
        #post_mcrun_funktions(var, mcvar, msg) # contains functions that get executed after mcstas finished and can i.e. reformate the output
        print('simulation successfully\n')

        return mcvar, res

def print_mcvariable_from_instrument(instrument):
    for line in create_class_mcvariables_lines(instrument):
        print(line)

def initialize(instrument='', working_dir=os.getcwd(), mcstas='mcstas', output_dir='simulation_results', component_dir='', mpi=0):
    if os.name == 'nt':
        working_dir   = re.sub(r'\\','/', working_dir)
        mcstas        = re.sub(r'\\','/', mcstas)
        instrument    = re.sub(r'\\','/', instrument)
        output_dir    = re.sub(r'\\','/', output_dir)
        component_dir = re.sub(r'\\','/', component_dir)

    instr_file = working_dir + "/" + instrument
    if not isfile(instr_file):
        sys.exit(f" the instrument file '{instrument}' is not located in the working directory '{working_dir}'")
    if not instrument.split('.')[1] == "instr":
        sys.exit(f"the given instrument file has not the correct ending:\
                 \nexpected: {instrument.split('.')[0]}.instr\
                 \ngot: {instrument}\
                 \nplease make shure to give a valid instrument file")

    if not isdir(working_dir):
        sys.exit(f"the given working directory '{working_dir}' dose not exist")
    if isabs(output_dir):
        if not isdir(output_dir):
            sys.exit(f"the given simulation result directory '{output_dir}' dose not exist")
    else:
        if not isdir(f"{working_dir}/{output_dir}"):
            os.mkdir(f"{working_dir}/{output_dir}")
            print(f"created simulation result directory '{output_dir}' in the working directory")

    if os.name=='nt':
        which(f"{mcstas} -v")
    else:
        which(mcstas)
    if mpi == 0:
        # test if gcc exists
        if os.name=='nt':
            which("gcc --help")
        else:
            which("gcc")
    else:
        # test if mpicc exists
        if os.name=='nt':
            pass
        else:
            which("mpicc")

    args= argparse.Namespace(working_dir=working_dir,\
                            mcstas=mcstas,\
                            instrument=instrument,\
                            output_dir=output_dir,\
                            component_dir=component_dir,\
                            mpi = mpi)

    if not os.path.isfile(f"{working_dir}/local_var.py"):
        create_local_var(args)

    sys.path.append(working_dir)
    from local_var import variables
    var = variables()
    valid_config(var)

    print(f'####################\ncopy the following part and execute it in a new cell\n####################\n')
    print("from mcpw.mcstas_wrapper import Scan\n")
    for line in create_class_mcvariables_lines(instr_file):
        print(line)
    print("mcvar = mcvariables()\n")
    print(f'####################\nend of cell\n####################')

    return var
